@@ -8,6 +8,11 @@
  * file that was distributed with this source code.
  */
 
+use SebastianBergmann\CodeCoverage\CodeCoverage;
+use SebastianBergmann\CodeCoverage\Exception as CodeCoverageException;
+use SebastianBergmann\CodeCoverage\CoveredCodeNotExecutedException;
+use SebastianBergmann\CodeCoverage\MissingCoversAnnotationException;
+use SebastianBergmann\CodeCoverage\UnintentionallyCoveredCodeException;
 use SebastianBergmann\ResourceOperations\ResourceOperations;
 
 /**
@@ -75,7 +80,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * Code Coverage information.
      *
-     * @var PHP_CodeCoverage
+     * @var CodeCoverage
      */
     protected $codeCoverage;
 
@@ -164,6 +169,11 @@ class PHPUnit_Framework_TestResult implements Countable
      */
     protected $lastTestFailed = false;
 
+    /**
+     * @var bool
+     */
+    private $registerMockObjectsFromTestArgumentsRecursively = false;
+
     /**
      * Registers a TestListener.
      *
@@ -191,7 +201,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * Flushes all flushable TestListeners.
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function flushListeners()
     {
@@ -337,7 +347,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @param PHPUnit_Framework_TestSuite $suite
      *
-     * @since  Method available since Release 2.2.0
+     * @since Method available since Release 2.2.0
      */
     public function startTestSuite(PHPUnit_Framework_TestSuite $suite)
     {
@@ -355,7 +365,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @param PHPUnit_Framework_TestSuite $suite
      *
-     * @since  Method available since Release 2.2.0
+     * @since Method available since Release 2.2.0
      */
     public function endTestSuite(PHPUnit_Framework_TestSuite $suite)
     {
@@ -412,7 +422,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return bool
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function allHarmless()
     {
@@ -424,7 +434,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return int
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function riskyCount()
     {
@@ -456,7 +466,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return array
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function risky()
     {
@@ -478,7 +488,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return bool
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function noneSkipped()
     {
@@ -490,7 +500,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return int
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function skippedCount()
     {
@@ -502,7 +512,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return array
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function skipped()
     {
@@ -554,7 +564,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return int
      *
-     * @since  Method available since Release 5.1.0
+     * @since Method available since Release 5.1.0
      */
     public function warningCount()
     {
@@ -566,7 +576,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return array
      *
-     * @since  Method available since Release 5.1.0
+     * @since Method available since Release 5.1.0
      */
     public function warnings()
     {
@@ -578,7 +588,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public function passed()
     {
@@ -590,7 +600,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return PHPUnit_Framework_TestSuite
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function topTestSuite()
     {
@@ -602,7 +612,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return bool If code coverage should be collected
      *
-     * @since  Method available since Release 3.2.0
+     * @since Method available since Release 3.2.0
      */
     public function getCollectCodeCoverageInformation()
     {
@@ -618,6 +628,12 @@ class PHPUnit_Framework_TestResult implements Countable
     {
         PHPUnit_Framework_Assert::resetCount();
 
+        if ($test instanceof PHPUnit_Framework_TestCase) {
+            $test->setRegisterMockObjectsFromTestArgumentsRecursively(
+                $this->registerMockObjectsFromTestArgumentsRecursively
+            );
+        }
+
         $error      = false;
         $failure    = false;
         $warning    = false;
@@ -684,6 +700,12 @@ class PHPUnit_Framework_TestResult implements Countable
             } else {
                 $test->runBare();
             }
+        } catch (PHPUnit_Framework_MockObject_Exception $e) {
+            $e = new PHPUnit_Framework_Warning(
+                $e->getMessage()
+            );
+
+            $warning = true;
         } catch (PHPUnit_Framework_AssertionFailedError $e) {
             $failure = true;
 
@@ -743,15 +765,25 @@ class PHPUnit_Framework_TestResult implements Countable
             $linesToBeUsed    = [];
 
             if ($append && $test instanceof PHPUnit_Framework_TestCase) {
-                $linesToBeCovered = PHPUnit_Util_Test::getLinesToBeCovered(
-                    get_class($test),
-                    $test->getName(false)
-                );
+                try {
+                    $linesToBeCovered = PHPUnit_Util_Test::getLinesToBeCovered(
+                        get_class($test),
+                        $test->getName(false)
+                    );
 
-                $linesToBeUsed = PHPUnit_Util_Test::getLinesToBeUsed(
-                    get_class($test),
-                    $test->getName(false)
-                );
+                    $linesToBeUsed = PHPUnit_Util_Test::getLinesToBeUsed(
+                        get_class($test),
+                        $test->getName(false)
+                    );
+                } catch (PHPUnit_Framework_InvalidCoversTargetException $cce) {
+                    $this->addWarning(
+                        $test,
+                        new PHPUnit_Framework_Warning(
+                            $cce->getMessage()
+                        ),
+                        $time
+                    );
+                }
             }
 
             try {
@@ -760,24 +792,35 @@ class PHPUnit_Framework_TestResult implements Countable
                     $linesToBeCovered,
                     $linesToBeUsed
                 );
-            } catch (PHP_CodeCoverage_UnintentionallyCoveredCodeException $cce) {
+            } catch (UnintentionallyCoveredCodeException $cce) {
+                if (!$test->isMedium() && !$test->isLarge()) {
+                    $this->addFailure(
+                        $test,
+                        new PHPUnit_Framework_UnintentionallyCoveredCodeError(
+                            'This test executed code that is not listed as code to be covered or used:' .
+                            PHP_EOL . $cce->getMessage()
+                        ),
+                        $time
+                    );
+                }
+            } catch (CoveredCodeNotExecutedException $cce) {
                 $this->addFailure(
                     $test,
-                    new PHPUnit_Framework_UnintentionallyCoveredCodeError(
-                        'This test executed code that is not listed as code to be covered or used:' .
+                    new PHPUnit_Framework_CoveredCodeNotExecutedException(
+                        'This test did not execute all the code that is listed as code to be covered:' .
                         PHP_EOL . $cce->getMessage()
                     ),
                     $time
                 );
-            } catch (PHPUnit_Framework_InvalidCoversTargetException $cce) {
+            } catch (MissingCoversAnnotationException $cce) {
                 $this->addFailure(
                     $test,
-                    new PHPUnit_Framework_InvalidCoversTargetError(
-                        $cce->getMessage()
+                    new PHPUnit_Framework_MissingCoversAnnotationException(
+                        'This test does not have a @covers annotation but is expected to have one'
                     ),
                     $time
                 );
-            } catch (PHP_CodeCoverage_Exception $cce) {
+            } catch (CodeCoverageException $cce) {
                 $error = true;
 
                 if (!isset($e)) {
@@ -862,11 +905,11 @@ class PHPUnit_Framework_TestResult implements Countable
     }
 
     /**
-     * Returns the PHP_CodeCoverage object.
+     * Returns the code coverage object.
      *
-     * @return PHP_CodeCoverage
+     * @return CodeCoverage
      *
-     * @since  Method available since Release 3.5.0
+     * @since Method available since Release 3.5.0
      */
     public function getCodeCoverage()
     {
@@ -874,13 +917,13 @@ class PHPUnit_Framework_TestResult implements Countable
     }
 
     /**
-     * Sets the PHP_CodeCoverage object.
+     * Sets the code coverage object.
      *
-     * @param PHP_CodeCoverage $codeCoverage
+     * @param CodeCoverage $codeCoverage
      *
      * @since Method available since Release 3.6.0
      */
-    public function setCodeCoverage(PHP_CodeCoverage $codeCoverage)
+    public function setCodeCoverage(CodeCoverage $codeCoverage)
     {
         $this->codeCoverage = $codeCoverage;
     }
@@ -892,7 +935,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.2.14
+     * @since Method available since Release 3.2.14
      */
     public function convertErrorsToExceptions($flag)
     {
@@ -908,7 +951,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @return bool
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public function getConvertErrorsToExceptions()
     {
@@ -922,7 +965,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.5.0
+     * @since Method available since Release 3.5.0
      */
     public function stopOnError($flag)
     {
@@ -940,7 +983,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.1.0
+     * @since Method available since Release 3.1.0
      */
     public function stopOnFailure($flag)
     {
@@ -958,7 +1001,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 5.1.0
+     * @since Method available since Release 5.1.0
      */
     public function stopOnWarning($flag)
     {
@@ -974,7 +1017,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function beStrictAboutTestsThatDoNotTestAnything($flag)
     {
@@ -988,7 +1031,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * @return bool
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function isStrictAboutTestsThatDoNotTestAnything()
     {
@@ -1000,7 +1043,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function beStrictAboutOutputDuringTests($flag)
     {
@@ -1014,7 +1057,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * @return bool
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function isStrictAboutOutputDuringTests()
     {
@@ -1026,7 +1069,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 5.0.0
+     * @since Method available since Release 5.0.0
      */
     public function beStrictAboutResourceUsageDuringSmallTests($flag)
     {
@@ -1040,7 +1083,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * @return bool
      *
-     * @since  Method available since Release 5.0.0
+     * @since Method available since Release 5.0.0
      */
     public function isStrictAboutResourceUsageDuringSmallTests()
     {
@@ -1052,7 +1095,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 5.0.0
+     * @since Method available since Release 5.0.0
      */
     public function enforceTimeLimit($flag)
     {
@@ -1066,7 +1109,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * @return bool
      *
-     * @since  Method available since Release 5.0.0
+     * @since Method available since Release 5.0.0
      */
     public function enforcesTimeLimit()
     {
@@ -1078,7 +1121,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 4.2.0
+     * @since Method available since Release 4.2.0
      */
     public function beStrictAboutTodoAnnotatedTests($flag)
     {
@@ -1092,7 +1135,7 @@ class PHPUnit_Framework_TestResult implements Countable
     /**
      * @return bool
      *
-     * @since  Method available since Release 4.2.0
+     * @since Method available since Release 4.2.0
      */
     public function isStrictAboutTodoAnnotatedTests()
     {
@@ -1106,7 +1149,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public function stopOnRisky($flag)
     {
@@ -1124,7 +1167,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.5.0
+     * @since Method available since Release 3.5.0
      */
     public function stopOnIncomplete($flag)
     {
@@ -1142,7 +1185,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.1.0
+     * @since Method available since Release 3.1.0
      */
     public function stopOnSkipped($flag)
     {
@@ -1180,7 +1223,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public function setTimeoutForSmallTests($timeout)
     {
@@ -1198,7 +1241,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public function setTimeoutForMediumTests($timeout)
     {
@@ -1216,7 +1259,7 @@ class PHPUnit_Framework_TestResult implements Countable
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public function setTimeoutForLargeTests($timeout)
     {
@@ -1227,6 +1270,30 @@ class PHPUnit_Framework_TestResult implements Countable
         $this->timeoutForLargeTests = $timeout;
     }
 
+    /**
+     * Returns the set timeout for large tests.
+     *
+     * @return int
+     */
+    public function getTimeoutForLargeTests()
+    {
+        return $this->timeoutForLargeTests;
+    }
+
+    /**
+     * @param bool $flag
+     *
+     * @since Method available since Release 5.4.0
+     */
+    public function setRegisterMockObjectsFromTestArgumentsRecursively($flag)
+    {
+        if (!is_bool($flag)) {
+            throw PHPUnit_Util_InvalidArgumentHelper::factory(1, 'boolean');
+        }
+
+        $this->registerMockObjectsFromTestArgumentsRecursively = $flag;
+    }
+
     /**
      * Returns the class hierarchy for a given class.
      *
