@@ -249,6 +249,10 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
                     $this->groups[$group][] = $test;
                 }
             }
+
+            if ($test instanceof PHPUnit_Framework_TestCase) {
+                $test->setGroups($groups);
+            }
         }
     }
 
@@ -313,7 +317,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 2.3.0
+     * @since Method available since Release 2.3.0
      */
     public function addTestFile($filename)
     {
@@ -339,7 +343,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
         // AFTER a child class that inherited from it. To account for that case,
         // cumulate all discovered classes, so the parent class may be found in
         // a later invocation.
-        if ($newClasses) {
+        if (!empty($newClasses)) {
             // On the assumption that test classes are defined first in files,
             // process discovered classes in approximate LIFO order, so as to
             // avoid unnecessary reflection.
@@ -393,7 +397,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 2.3.0
+     * @since Method available since Release 2.3.0
      */
     public function addTestFiles($filenames)
     {
@@ -419,13 +423,15 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      */
     public function count($preferCache = false)
     {
-        if ($preferCache && $this->cachedNumTests != null) {
+        if ($preferCache && $this->cachedNumTests !== null) {
             $numTests = $this->cachedNumTests;
         } else {
             $numTests = 0;
+
             foreach ($this as $test) {
                 $numTests += count($test);
             }
+
             $this->cachedNumTests = $numTests;
         }
 
@@ -638,7 +644,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.0
+     * @since Method available since Release 3.2.0
      */
     public function getGroups()
     {
@@ -765,7 +771,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.7.0
+     * @since Method available since Release 3.7.0
      */
     public function setRunTestInSeparateProcess($runTestInSeparateProcess)
     {
@@ -802,7 +808,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
     /**
      * Returns the test at the given index.
      *
-     * @param  int
+     * @param  int|false
      *
      * @return PHPUnit_Framework_Test
      */
@@ -844,7 +850,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @throws PHPUnit_Framework_SkippedTestSuiteError
      *
-     * @since  Method available since Release 3.0.0
+     * @since Method available since Release 3.0.0
      */
     public function markTestSuiteSkipped($message = '')
     {
@@ -905,10 +911,10 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
 
         // @scenario on TestCase::testMethod()
         // @test     on TestCase::testMethod()
-        $doc_comment = $method->getDocComment();
+        $docComment = $method->getDocComment();
 
-        return strpos($doc_comment, '@test')     !== false ||
-               strpos($doc_comment, '@scenario') !== false;
+        return strpos($docComment, '@test')     !== false ||
+               strpos($docComment, '@scenario') !== false;
     }
 
     /**
@@ -928,7 +934,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @return PHPUnit_Framework_SkippedTestCase
      *
-     * @since  Method available since Release 4.3.0
+     * @since Method available since Release 4.3.0
      */
     protected static function skipTest($class, $methodName, $message)
     {
@@ -942,7 +948,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @return PHPUnit_Framework_IncompleteTestCase
      *
-     * @since  Method available since Release 4.3.0
+     * @since Method available since Release 4.3.0
      */
     protected static function incompleteTest($class, $methodName, $message)
     {
@@ -952,7 +958,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
     /**
      * @param bool $beStrictAboutChangesToGlobalState
      *
-     * @since  Method available since Release 4.6.0
+     * @since Method available since Release 4.6.0
      */
     public function setbeStrictAboutChangesToGlobalState($beStrictAboutChangesToGlobalState)
     {
@@ -964,7 +970,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
     /**
      * @param bool $backupGlobals
      *
-     * @since  Method available since Release 3.3.0
+     * @since Method available since Release 3.3.0
      */
     public function setBackupGlobals($backupGlobals)
     {
@@ -976,7 +982,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
     /**
      * @param bool $backupStaticAttributes
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public function setBackupStaticAttributes($backupStaticAttributes)
     {
@@ -991,7 +997,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      *
      * @return RecursiveIteratorIterator
      *
-     * @since  Method available since Release 3.1.0
+     * @since Method available since Release 3.1.0
      */
     public function getIterator()
     {
@@ -1018,7 +1024,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      * Template Method that is called before the tests
      * of this test suite are run.
      *
-     * @since  Method available since Release 3.1.0
+     * @since Method available since Release 3.1.0
      */
     protected function setUp()
     {
@@ -1028,7 +1034,7 @@ class PHPUnit_Framework_TestSuite implements PHPUnit_Framework_Test, PHPUnit_Fra
      * Template Method that is called after the tests
      * of this test suite have finished running.
      *
-     * @since  Method available since Release 3.1.0
+     * @since Method available since Release 3.1.0
      */
     protected function tearDown()
     {
