@@ -30,7 +30,7 @@ class PHPUnit_Runner_Version
         }
 
         if (self::$version === null) {
-            $version       = new SebastianBergmann\Version('5.1.3', dirname(dirname(__DIR__)));
+            $version       = new SebastianBergmann\Version('5.4.6', dirname(dirname(__DIR__)));
             self::$version = $version->getVersion();
         }
 
@@ -44,7 +44,13 @@ class PHPUnit_Runner_Version
      */
     public static function series()
     {
-        return implode('.', array_slice(explode('.', self::id()), 0, 2));
+        if (strpos(self::id(), '-')) {
+            $version = explode('-', self::id())[0];
+        } else {
+            $version = self::id();
+        }
+
+        return implode('.', array_slice(explode('.', $version), 0, 2));
     }
 
     /**
@@ -58,16 +64,12 @@ class PHPUnit_Runner_Version
     /**
      * @return string
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public static function getReleaseChannel()
     {
-        if (strpos(self::$pharVersion, 'alpha') !== false) {
-            return '-alpha';
-        }
-
-        if (strpos(self::$pharVersion, 'beta') !== false) {
-            return '-beta';
+        if (strpos(self::$pharVersion, '-') !== false) {
+            return '-nightly';
         }
 
         return '';
