@@ -26,7 +26,6 @@
  *          convertNoticesToExceptions="true"
  *          convertWarningsToExceptions="true"
  *          forceCoversAnnotation="false"
- *          mapTestClassNameToCoveredClassName="false"
  *          printerClass="PHPUnit_TextUI_ResultPrinter"
  *          processIsolation="false"
  *          stopOnError="false"
@@ -35,8 +34,11 @@
  *          stopOnIncomplete="false"
  *          stopOnRisky="false"
  *          stopOnSkipped="false"
+ *          failOnWarning="false"
+ *          failOnRisky="false"
  *          testSuiteLoaderClass="PHPUnit_Runner_StandardTestSuiteLoader"
  *          beStrictAboutChangesToGlobalState="false"
+ *          beStrictAboutCoversAnnotation="false"
  *          beStrictAboutOutputDuringTests="false"
  *          beStrictAboutResourceUsageDuringSmallTests="false"
  *          beStrictAboutTestsThatDoNotTestAnything="false"
@@ -47,7 +49,8 @@
  *          timeoutForMediumTests="10"
  *          timeoutForLargeTests="60"
  *          verbose="false"
- *          reverseDefectList="false">
+ *          reverseDefectList="false"
+ *          registerMockObjectsFromTestArgumentsRecursively="false">
  *   <testsuites>
  *     <testsuite name="My Test Suite">
  *       <directory suffix="Test.php" phpVersion="5.3.0" phpVersionOperator=">=">/path/to/files</directory>
@@ -65,6 +68,15 @@
  *     </exclude>
  *   </groups>
  *
+ *   <testdoxGroups>
+ *     <include>
+ *       <group>name</group>
+ *     </include>
+ *     <exclude>
+ *       <group>name</group>
+ *     </exclude>
+ *   </testdoxGroups>
+ *
  *   <filter>
  *     <whitelist addUncoveredFilesFromWhitelist="true"
  *                processUncoveredFilesFromWhitelist="false">
@@ -104,9 +116,10 @@
  *     <log type="plain" target="/tmp/logfile.txt"/>
  *     <log type="tap" target="/tmp/logfile.tap"/>
  *     <log type="teamcity" target="/tmp/logfile.txt"/>
- *     <log type="junit" target="/tmp/logfile.xml" logIncompleteSkipped="false"/>
+ *     <log type="junit" target="/tmp/logfile.xml"/>
  *     <log type="testdox-html" target="/tmp/testdox.html"/>
  *     <log type="testdox-text" target="/tmp/testdox.txt"/>
+ *     <log type="testdox-xml" target="/tmp/testdox.xml"/>
  *   </logging>
  *
  *   <php>
@@ -148,7 +161,7 @@ class PHPUnit_Util_Configuration
     }
 
     /**
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     final private function __clone()
     {
@@ -161,7 +174,7 @@ class PHPUnit_Util_Configuration
      *
      * @return PHPUnit_Util_Configuration
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getInstance($filename)
     {
@@ -188,7 +201,7 @@ class PHPUnit_Util_Configuration
      *
      * @return string
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public function getFilename()
     {
@@ -200,7 +213,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.1
+     * @since Method available since Release 3.2.1
      */
     public function getFilterConfiguration()
     {
@@ -258,20 +271,42 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.1
+     * @since Method available since Release 3.2.1
      */
     public function getGroupConfiguration()
+    {
+        return $this->parseGroupConfiguration('groups');
+    }
+
+    /**
+     * Returns the configuration for testdox groups.
+     *
+     * @return array
+     *
+     * @since Method available since Release 5.4.0
+     */
+    public function getTestdoxGroupConfiguration()
+    {
+        return $this->parseGroupConfiguration('testdoxGroups');
+    }
+
+    /**
+     * @param string $root
+     *
+     * @return array
+     */
+    private function parseGroupConfiguration($root)
     {
         $groups = [
-          'include' => [],
-          'exclude' => []
+            'include' => [],
+            'exclude' => []
         ];
 
-        foreach ($this->xpath->query('groups/include/group') as $group) {
+        foreach ($this->xpath->query($root . '/include/group') as $group) {
             $groups['include'][] = (string) $group->textContent;
         }
 
-        foreach ($this->xpath->query('groups/exclude/group') as $group) {
+        foreach ($this->xpath->query($root . '/exclude/group') as $group) {
             $groups['exclude'][] = (string) $group->textContent;
         }
 
@@ -283,7 +318,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public function getListenerConfiguration()
     {
@@ -366,13 +401,6 @@ class PHPUnit_Util_Configuration
                         30
                     );
                 }
-            } elseif ($type == 'junit') {
-                if ($log->hasAttribute('logIncompleteSkipped')) {
-                    $result['logIncompleteSkipped'] = $this->getBoolean(
-                        (string) $log->getAttribute('logIncompleteSkipped'),
-                        false
-                    );
-                }
             } elseif ($type == 'coverage-text') {
                 if ($log->hasAttribute('showUncoveredFiles')) {
                     $result['coverageTextShowUncoveredFiles'] = $this->getBoolean(
@@ -399,7 +427,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.1
+     * @since Method available since Release 3.2.1
      */
     public function getPHPConfiguration()
     {
@@ -453,7 +481,7 @@ class PHPUnit_Util_Configuration
     /**
      * Handles the PHP configuration.
      *
-     * @since  Method available since Release 3.2.20
+     * @since Method available since Release 3.2.20
      */
     public function handlePHPConfiguration()
     {
@@ -518,7 +546,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.14
+     * @since Method available since Release 3.2.14
      */
     public function getPHPUnitConfiguration()
     {
@@ -610,9 +638,9 @@ class PHPUnit_Util_Configuration
             );
         }
 
-        if ($root->hasAttribute('mapTestClassNameToCoveredClassName')) {
-            $result['mapTestClassNameToCoveredClassName'] = $this->getBoolean(
-                (string) $root->getAttribute('mapTestClassNameToCoveredClassName'),
+        if ($root->hasAttribute('disableCodeCoverageIgnore')) {
+            $result['disableCodeCoverageIgnore'] = $this->getBoolean(
+                (string) $root->getAttribute('disableCodeCoverageIgnore'),
                 false
             );
         }
@@ -666,6 +694,20 @@ class PHPUnit_Util_Configuration
             );
         }
 
+        if ($root->hasAttribute('failOnWarning')) {
+            $result['failOnWarning'] = $this->getBoolean(
+                (string) $root->getAttribute('failOnWarning'),
+                false
+            );
+        }
+
+        if ($root->hasAttribute('failOnRisky')) {
+            $result['failOnRisky'] = $this->getBoolean(
+                (string) $root->getAttribute('failOnRisky'),
+                false
+            );
+        }
+
         if ($root->hasAttribute('testSuiteLoaderClass')) {
             $result['testSuiteLoaderClass'] = (string) $root->getAttribute(
                 'testSuiteLoaderClass'
@@ -725,11 +767,18 @@ class PHPUnit_Util_Configuration
             );
         }
 
-        if ($root->hasAttribute('checkForUnintentionallyCoveredCode')) {
+        if ($root->hasAttribute('beStrictAboutCoversAnnotation')) {
+            $result['strictCoverage'] = $this->getBoolean(
+                (string) $root->getAttribute('beStrictAboutCoversAnnotation'),
+                false
+            );
+        } elseif ($root->hasAttribute('checkForUnintentionallyCoveredCode')) {
             $result['strictCoverage'] = $this->getBoolean(
                 (string) $root->getAttribute('checkForUnintentionallyCoveredCode'),
                 false
             );
+
+            $result['deprecatedCheckForUnintentionallyCoveredCodeSettingUsed'] = true;
         }
 
         if ($root->hasAttribute('enforceTimeLimit')) {
@@ -774,6 +823,13 @@ class PHPUnit_Util_Configuration
             );
         }
 
+        if ($root->hasAttribute('registerMockObjectsFromTestArgumentsRecursively')) {
+            $result['registerMockObjectsFromTestArgumentsRecursively'] = $this->getBoolean(
+                (string) $root->getAttribute('registerMockObjectsFromTestArgumentsRecursively'),
+                false
+            );
+        }
+
         return $result;
     }
 
@@ -782,7 +838,7 @@ class PHPUnit_Util_Configuration
      *
      * @return PHPUnit_Framework_TestSuite
      *
-     * @since  Method available since Release 3.2.1
+     * @since Method available since Release 3.2.1
      */
     public function getTestSuiteConfiguration($testSuiteFilter = null)
     {
@@ -814,7 +870,7 @@ class PHPUnit_Util_Configuration
      *
      * @return PHPUnit_Framework_TestSuite
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     protected function getTestSuite(DOMElement $testSuiteNode, $testSuiteFilter = null)
     {
@@ -935,7 +991,7 @@ class PHPUnit_Util_Configuration
      *
      * @return bool
      *
-     * @since  Method available since Release 3.2.3
+     * @since Method available since Release 3.2.3
      */
     protected function getBoolean($value, $default)
     {
@@ -954,7 +1010,7 @@ class PHPUnit_Util_Configuration
      *
      * @return bool
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     protected function getInteger($value, $default)
     {
@@ -970,7 +1026,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.3
+     * @since Method available since Release 3.2.3
      */
     protected function readFilterDirectories($query)
     {
@@ -1017,7 +1073,7 @@ class PHPUnit_Util_Configuration
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.3
+     * @since Method available since Release 3.2.3
      */
     protected function readFilterFiles($query)
     {
@@ -1025,6 +1081,7 @@ class PHPUnit_Util_Configuration
 
         foreach ($this->xpath->query($query) as $file) {
             $filePath = (string) $file->textContent;
+
             if ($filePath) {
                 $files[] = $this->toAbsolutePath($filePath);
             }
@@ -1039,10 +1096,12 @@ class PHPUnit_Util_Configuration
      *
      * @return string
      *
-     * @since  Method available since Release 3.5.0
+     * @since Method available since Release 3.5.0
      */
     protected function toAbsolutePath($path, $useIncludePath = false)
     {
+        $path = trim($path);
+
         if ($path[0] === '/') {
             return $path;
         }
