@@ -27,6 +27,26 @@ abstract class PHPUnit_Util_PHP
      */
     protected $stderrRedirection = false;
 
+    /**
+     * @var string
+     */
+    protected $stdin = '';
+
+    /**
+     * @var string
+     */
+    protected $args = '';
+
+    /**
+     * @var array
+     */
+    protected $env = [];
+
+    /**
+     * @var int
+     */
+    protected $timeout = 0;
+
     /**
      * Creates internal Runtime instance.
      */
@@ -63,10 +83,90 @@ abstract class PHPUnit_Util_PHP
         return $this->stderrRedirection;
     }
 
+    /**
+     * Sets the input string to be sent via STDIN
+     *
+     * @param string $stdin
+     */
+    public function setStdin($stdin)
+    {
+        $this->stdin = (string) $stdin;
+    }
+
+    /**
+     * Returns the input string to be sent via STDIN
+     *
+     * @return string
+     */
+    public function getStdin()
+    {
+        return $this->stdin;
+    }
+
+    /**
+     * Sets the string of arguments to pass to the php job
+     *
+     * @param string $args
+     */
+    public function setArgs($args)
+    {
+        $this->args = (string) $args;
+    }
+
+    /**
+     * Returns the string of arguments to pass to the php job
+     *
+     * @retrun string
+     */
+    public function getArgs()
+    {
+        return $this->args;
+    }
+
+    /**
+     * Sets the array of environment variables to start the child process with
+     *
+     * @param array $env
+     */
+    public function setEnv(array $env)
+    {
+        $this->env = $env;
+    }
+
+    /**
+     * Returns the array of environment variables to start the child process with
+     *
+     * @return array
+     */
+    public function getEnv()
+    {
+        return $this->env;
+    }
+
+    /**
+     * Sets the amount of seconds to wait before timing out
+     *
+     * @param int $timeout
+     */
+    public function setTimeout($timeout)
+    {
+        $this->timeout = (int) $timeout;
+    }
+
+    /**
+     * Returns the amount of seconds to wait before timing out
+     *
+     * @return int
+     */
+    public function getTimeout()
+    {
+        return $this->timeout;
+    }
+
     /**
      * @return PHPUnit_Util_PHP
      *
-     * @since  Method available since Release 3.5.12
+     * @since Method available since Release 3.5.12
      */
     public static function factory()
     {
@@ -103,18 +203,32 @@ abstract class PHPUnit_Util_PHP
     /**
      * Returns the command based into the configurations.
      *
-     * @param array $settings
+     * @param array       $settings
+     * @param string|null $file
      *
      * @return string
      */
-    public function getCommand(array $settings)
+    public function getCommand(array $settings, $file = null)
     {
         $command = $this->runtime->getBinary();
         $command .= $this->settingsToParameters($settings);
 
         if ('phpdbg' === PHP_SAPI) {
-            $command .= ' -qrr ' . escapeshellarg(__DIR__ . '/PHP/eval-stdin.php');
+            $command .= ' -qrr ';
+
+            if ($file) {
+                $command .= '-e ' . escapeshellarg($file);
+            } else {
+                $command .= escapeshellarg(__DIR__ . '/PHP/eval-stdin.php');
+            }
+        } elseif ($file) {
+            $command .= ' -f ' . escapeshellarg($file);
         }
+
+        if ($this->args) {
+            $command .= ' -- ' . $this->args;
+        }
+
         if (true === $this->stderrRedirection) {
             $command .= ' 2>&1';
         }
@@ -265,7 +379,7 @@ abstract class PHPUnit_Util_PHP
      *
      * @return Exception
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      * @see    https://github.com/sebastianbergmann/phpunit/issues/74
      */
     private function getException(PHPUnit_Framework_TestFailure $error)
