@@ -18,9 +18,9 @@ class PHPUnit_Util_Test
     const REGEX_DATA_PROVIDER      = '/@dataProvider\s+([a-zA-Z0-9._:-\\\\x7f-\xff]+)/';
     const REGEX_TEST_WITH          = '/@testWith\s+/';
     const REGEX_EXPECTED_EXCEPTION = '(@expectedException\s+([:.\w\\\\x7f-\xff]+)(?:[\t ]+(\S*))?(?:[\t ]+(\S*))?\s*$)m';
-    const REGEX_REQUIRES_VERSION   = '/@requires\s+(?P<name>PHP(?:Unit)?)\s+(?P<value>[\d\.-]+(dev|(RC|alpha|beta)[\d\.])?)[ \t]*\r?$/m';
+    const REGEX_REQUIRES_VERSION   = '/@requires\s+(?P<name>PHP(?:Unit)?)\s+(?P<operator>[<>=!]{0,2})\s*(?P<version>[\d\.-]+(dev|(RC|alpha|beta)[\d\.])?)[ \t]*\r?$/m';
     const REGEX_REQUIRES_OS        = '/@requires\s+OS\s+(?P<value>.+?)[ \t]*\r?$/m';
-    const REGEX_REQUIRES           = '/@requires\s+(?P<name>function|extension)\s+(?P<value>([^ ]+?))[ \t]*\r?$/m';
+    const REGEX_REQUIRES           = '/@requires\s+(?P<name>function|extension)\s+(?P<value>([^ ]+?))\s*(?P<operator>[<>=!]{0,2})\s*(?P<version>[\d\.-]+[\d\.]?)?[ \t]*\r?$/m';
 
     const UNKNOWN = -1;
     const SMALL   = 0;
@@ -66,7 +66,7 @@ class PHPUnit_Util_Test
      *
      * @throws PHPUnit_Framework_CodeCoverageException
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public static function getLinesToBeCovered($className, $methodName)
     {
@@ -90,7 +90,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     public static function getLinesToBeUsed($className, $methodName)
     {
@@ -106,7 +106,7 @@ class PHPUnit_Util_Test
      *
      * @throws PHPUnit_Framework_CodeCoverageException
      *
-     * @since  Method available since Release 4.2.0
+     * @since Method available since Release 4.2.0
      */
     private static function getLinesToBeCoveredOrUsed($className, $methodName, $mode)
     {
@@ -169,7 +169,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public static function getRequirements($className, $methodName)
     {
@@ -187,7 +187,10 @@ class PHPUnit_Util_Test
         }
         if ($count = preg_match_all(self::REGEX_REQUIRES_VERSION, $docComment, $matches)) {
             for ($i = 0; $i < $count; $i++) {
-                $requires[$matches['name'][$i]] = $matches['value'][$i];
+                $requires[$matches['name'][$i]] = [
+                    'version'  => $matches['version'][$i],
+                    'operator' => $matches['operator'][$i]
+                ];
             }
         }
 
@@ -201,6 +204,13 @@ class PHPUnit_Util_Test
                     $requires[$name] = [];
                 }
                 $requires[$name][] = $matches['value'][$i];
+                if (empty($matches['version'][$i]) || $name != 'extensions') {
+                    continue;
+                }
+                $requires['extension_versions'][$matches['value'][$i]] = [
+                    'version'  => $matches['version'][$i],
+                    'operator' => $matches['operator'][$i]
+                ];
             }
         }
 
@@ -215,21 +225,24 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 4.3.0
+     * @since Method available since Release 4.3.0
      */
     public static function getMissingRequirements($className, $methodName)
     {
         $required = static::getRequirements($className, $methodName);
         $missing  = [];
 
-        if (!empty($required['PHP']) && version_compare(PHP_VERSION, $required['PHP'], '<')) {
-            $missing[] = sprintf('PHP %s (or later) is required.', $required['PHP']);
+        $operator = empty($required['PHP']['operator']) ? '>=' : $required['PHP']['operator'];
+        if (!empty($required['PHP']) && !version_compare(PHP_VERSION, $required['PHP']['version'], $operator)) {
+            $missing[] = sprintf('PHP %s %s is required.', $operator, $required['PHP']['version']);
         }
 
         if (!empty($required['PHPUnit'])) {
             $phpunitVersion = PHPUnit_Runner_Version::id();
-            if (version_compare($phpunitVersion, $required['PHPUnit'], '<')) {
-                $missing[] = sprintf('PHPUnit %s (or later) is required.', $required['PHPUnit']);
+
+            $operator = empty($required['PHPUnit']['operator']) ? '>=' : $required['PHPUnit']['operator'];
+            if (!version_compare($phpunitVersion, $required['PHPUnit']['version'], $operator)) {
+                $missing[] = sprintf('PHPUnit %s %s is required.', $operator, $required['PHPUnit']['version']);
             }
         }
 
@@ -252,12 +265,26 @@ class PHPUnit_Util_Test
 
         if (!empty($required['extensions'])) {
             foreach ($required['extensions'] as $extension) {
+                if (isset($required['extension_versions'][$extension])) {
+                    continue;
+                }
                 if (!extension_loaded($extension)) {
                     $missing[] = sprintf('Extension %s is required.', $extension);
                 }
             }
         }
 
+        if (!empty($required['extension_versions'])) {
+            foreach ($required['extension_versions'] as $extension => $required) {
+                $actualVersion = phpversion($extension);
+
+                $operator = empty($required['operator']) ? '>=' : $required['operator'];
+                if (false === $actualVersion || !version_compare($actualVersion, $required['version'], $operator)) {
+                    $missing[] = sprintf('Extension %s %s %s is required.', $extension, $operator, $required['version']);
+                }
+            }
+        }
+
         return $missing;
     }
 
@@ -269,7 +296,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.3.6
+     * @since Method available since Release 3.3.6
      */
     public static function getExpectedException($className, $methodName)
     {
@@ -357,7 +384,7 @@ class PHPUnit_Util_Test
      *
      * @throws PHPUnit_Framework_Exception
      *
-     * @since  Method available since Release 3.2.0
+     * @since Method available since Release 3.2.0
      */
     public static function getProvidedData($className, $methodName)
     {
@@ -495,7 +522,7 @@ class PHPUnit_Util_Test
      *
      * @throws ReflectionException
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function parseTestMethodAnnotations($className, $methodName = '')
     {
@@ -520,12 +547,44 @@ class PHPUnit_Util_Test
         ];
     }
 
+    /**
+     * @param string $className
+     * @param string $methodName
+     *
+     * @return array
+     *
+     * @since Method available since Release 5.4.0
+     */
+    public static function getInlineAnnotations($className, $methodName)
+    {
+        $method      = new ReflectionMethod($className, $methodName);
+        $code        = file($method->getFileName());
+        $lineNumber  = $method->getStartLine();
+        $startLine   = $method->getStartLine() - 1;
+        $endLine     = $method->getEndLine() - 1;
+        $methodLines = array_slice($code, $startLine, $endLine - $startLine + 1);
+        $annotations = [];
+
+        foreach ($methodLines as $line) {
+            if (preg_match('#/\*\*?\s*@(?P<name>[A-Za-z_-]+)(?:[ \t]+(?P<value>.*?))?[ \t]*\r?\*/$#m', $line, $matches)) {
+                $annotations[strtolower($matches['name'])] = [
+                    'line'  => $lineNumber,
+                    'value' => $matches['value']
+                ];
+            }
+
+            $lineNumber++;
+        }
+
+        return $annotations;
+    }
+
     /**
      * @param string $docblock
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     private static function parseAnnotations($docblock)
     {
@@ -552,7 +611,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getBackupSettings($className, $methodName)
     {
@@ -578,7 +637,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getDependencies($className, $methodName)
     {
@@ -611,7 +670,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getErrorHandlerSettings($className, $methodName)
     {
@@ -630,7 +689,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.2.0
+     * @since Method available since Release 3.2.0
      */
     public static function getGroups($className, $methodName = '')
     {
@@ -669,11 +728,6 @@ class PHPUnit_Util_Test
                     $groups[] = $size;
                     break 2;
                 }
-
-                if (isset($annotations[$element][$size])) {
-                    $groups[] = $size;
-                    break 2;
-                }
             }
         }
 
@@ -688,7 +742,7 @@ class PHPUnit_Util_Test
      *
      * @return int
      *
-     * @since  Method available since Release 3.6.0
+     * @since Method available since Release 3.6.0
      */
     public static function getSize($className, $methodName)
     {
@@ -717,7 +771,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getTickets($className, $methodName)
     {
@@ -747,7 +801,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 3.4.1
+     * @since Method available since Release 3.4.1
      */
     public static function getProcessIsolationSettings($className, $methodName)
     {
@@ -772,7 +826,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     public static function getPreserveGlobalStateSettings($className, $methodName)
     {
@@ -788,7 +842,7 @@ class PHPUnit_Util_Test
      *
      * @return array
      *
-     * @since  Method available since Release 4.0.8
+     * @since Method available since Release 4.0.8
      */
     public static function getHookMethods($className)
     {
@@ -829,7 +883,7 @@ class PHPUnit_Util_Test
     /**
      * @return array
      *
-     * @since  Method available since Release 4.0.9
+     * @since Method available since Release 4.0.9
      */
     private static function emptyHookMethodsArray()
     {
@@ -848,7 +902,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 3.4.0
+     * @since Method available since Release 3.4.0
      */
     private static function getBooleanAnnotationSetting($className, $methodName, $settingName)
     {
@@ -885,7 +939,7 @@ class PHPUnit_Util_Test
      *
      * @throws PHPUnit_Framework_InvalidCoversTargetException
      *
-     * @since  Method available since Release 4.0.0
+     * @since Method available since Release 4.0.0
      */
     private static function resolveElementToReflectionObjects($element)
     {
@@ -1031,7 +1085,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 4.0.8
+     * @since Method available since Release 4.0.8
      */
     private static function isBeforeClassMethod(ReflectionMethod $method)
     {
@@ -1043,7 +1097,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 4.0.8
+     * @since Method available since Release 4.0.8
      */
     private static function isBeforeMethod(ReflectionMethod $method)
     {
@@ -1055,7 +1109,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 4.0.8
+     * @since Method available since Release 4.0.8
      */
     private static function isAfterClassMethod(ReflectionMethod $method)
     {
@@ -1067,7 +1121,7 @@ class PHPUnit_Util_Test
      *
      * @return bool
      *
-     * @since  Method available since Release 4.0.8
+     * @since Method available since Release 4.0.8
      */
     private static function isAfterMethod(ReflectionMethod $method)
     {
