@@ -10,6 +10,7 @@
 
 require_once dirname(__DIR__) . DIRECTORY_SEPARATOR . '_files' . DIRECTORY_SEPARATOR . 'NoArgTestCaseTest.php';
 require_once dirname(__DIR__) . DIRECTORY_SEPARATOR . '_files' . DIRECTORY_SEPARATOR . 'Singleton.php';
+require_once dirname(__DIR__) . DIRECTORY_SEPARATOR . '_files' . DIRECTORY_SEPARATOR . 'Mockable.php';
 
 $GLOBALS['a']  = 'a';
 $_ENV['b']     = 'b';
@@ -195,6 +196,28 @@ class Framework_TestCaseTest extends PHPUnit_Framework_TestCase
         $this->assertTrue($result->wasSuccessful());
     }
 
+    public function testExceptionWithEmptyMessage()
+    {
+        $test = new ThrowExceptionTestCase('test');
+        $test->setExpectedException('RuntimeException', '');
+
+        $result = $test->run();
+
+        $this->assertEquals(1, count($result));
+        $this->assertTrue($result->wasSuccessful());
+    }
+
+    public function testExceptionWithNullMessage()
+    {
+        $test = new ThrowExceptionTestCase('test');
+        $test->setExpectedException('RuntimeException', null);
+
+        $result = $test->run();
+
+        $this->assertEquals(1, count($result));
+        $this->assertTrue($result->wasSuccessful());
+    }
+
     public function testExceptionWithMessage()
     {
         $test = new ThrowExceptionTestCase('test');
@@ -431,7 +454,7 @@ class Framework_TestCaseTest extends PHPUnit_Framework_TestCase
 
         $this->assertEquals(1, $result->skippedCount());
         $this->assertEquals(
-            'PHPUnit 1111111 (or later) is required.',
+            'PHPUnit >= 1111111 is required.',
             $test->getStatusMessage()
         );
     }
@@ -443,7 +466,7 @@ class Framework_TestCaseTest extends PHPUnit_Framework_TestCase
 
         $this->assertEquals(1, $result->skippedCount());
         $this->assertEquals(
-            'PHP 9999999 (or later) is required.',
+            'PHP >= 9999999 is required.',
             $test->getStatusMessage()
         );
     }
@@ -483,19 +506,31 @@ class Framework_TestCaseTest extends PHPUnit_Framework_TestCase
         );
     }
 
+    public function testSkipsIfRequiresExtensionWithAMinimumVersion()
+    {
+        $test   = new RequirementsTest('testSpecificExtensionVersion');
+        $result = $test->run();
+
+        $this->assertEquals(
+            'Extension testExt >= 1.8.0 is required.',
+            $test->getStatusMessage()
+        );
+    }
+
     public function testSkipsProvidesMessagesForAllSkippingReasons()
     {
         $test   = new RequirementsTest('testAllPossibleRequirements');
         $result = $test->run();
 
         $this->assertEquals(
-            'PHP 99-dev (or later) is required.' . PHP_EOL .
-            'PHPUnit 9-dev (or later) is required.' . PHP_EOL .
+            'PHP >= 99-dev is required.' . PHP_EOL .
+            'PHPUnit >= 9-dev is required.' . PHP_EOL .
             'Operating system matching /DOESNOTEXIST/i is required.' . PHP_EOL .
             'Function testFuncOne is required.' . PHP_EOL .
             'Function testFuncTwo is required.' . PHP_EOL .
             'Extension testExtOne is required.' . PHP_EOL .
-            'Extension testExtTwo is required.',
+            'Extension testExtTwo is required.' . PHP_EOL .
+            'Extension testExtThree >= 2.0 is required.',
             $test->getStatusMessage()
         );
     }
@@ -547,4 +582,37 @@ class Framework_TestCaseTest extends PHPUnit_Framework_TestCase
         $o = new ClassWithScalarTypeDeclarations;
         $o->foo(null, null);
     }
+
+    public function testCreateMockFromClassName()
+    {
+        $mock = $this->createMock(Mockable::class);
+
+        $this->assertInstanceOf(Mockable::class, $mock);
+        $this->assertInstanceOf(PHPUnit_Framework_MockObject_MockObject::class, $mock);
+    }
+
+    public function testCreateMockMocksAllMethods()
+    {
+        /** @var Mockable $mock */
+        $mock = $this->createMock(Mockable::class);
+
+        $this->assertNull($mock->foo());
+    }
+
+    public function testCreateMockSkipsConstructor()
+    {
+        /** @var Mockable $mock */
+        $mock = $this->createMock(Mockable::class);
+
+        $this->assertFalse($mock->constructorCalled);
+    }
+
+    public function testCreateMockDisablesOriginalClone()
+    {
+        /** @var Mockable $mock */
+        $mock = $this->createMock(Mockable::class);
+
+        $cloned = clone $mock;
+        $this->assertFalse($cloned->cloned);
+    }
 }
