@@ -135,13 +135,16 @@ class Util_TestTest extends PHPUnit_Framework_TestCase
     {
         return [
             ['testOne',    []],
-            ['testTwo',    ['PHPUnit'    => '1.0']],
-            ['testThree',  ['PHP'        => '2.0']],
-            ['testFour',   ['PHPUnit'    => '2.0', 'PHP' => '1.0']],
-            ['testFive',   ['PHP'        => '5.4.0RC6']],
-            ['testSix',    ['PHP'        => '5.4.0-alpha1']],
-            ['testSeven',  ['PHP'        => '5.4.0beta2']],
-            ['testEight',  ['PHP'        => '5.4-dev']],
+            ['testTwo',    ['PHPUnit'    => ['version' => '1.0', 'operator' => '']]],
+            ['testThree',  ['PHP'        => ['version' => '2.0', 'operator' => '']]],
+            ['testFour',   [
+                'PHPUnit'    => ['version' => '2.0', 'operator' => ''],
+                'PHP'        => ['version' => '1.0', 'operator' => '']]
+            ],
+            ['testFive',   ['PHP'        => ['version' => '5.4.0RC6', 'operator' => '']]],
+            ['testSix',    ['PHP'        => ['version' => '5.4.0-alpha1', 'operator' => '']]],
+            ['testSeven',  ['PHP'        => ['version' => '5.4.0beta2', 'operator' => '']]],
+            ['testEight',  ['PHP'        => ['version' => '5.4-dev', 'operator' => '']]],
             ['testNine',   ['functions'  => ['testFunc']]],
             ['testTen',    ['extensions' => ['testExt']]],
             ['testEleven', ['OS'         => '/Linux/i']],
@@ -155,8 +158,8 @@ class Util_TestTest extends PHPUnit_Framework_TestCase
             [
               'testAllPossibleRequirements',
               [
-                'PHP'       => '99-dev',
-                'PHPUnit'   => '9-dev',
+                'PHP'       => ['version' => '99-dev', 'operator' => ''],
+                'PHPUnit'   => ['version' => '9-dev', 'operator' => ''],
                 'OS'        => '/DOESNOTEXIST/i',
                 'functions' => [
                   'testFuncOne',
@@ -165,9 +168,157 @@ class Util_TestTest extends PHPUnit_Framework_TestCase
                 'extensions' => [
                   'testExtOne',
                   'testExtTwo',
+                  'testExtThree',
+                ],
+                'extension_versions' => [
+                    'testExtThree' => ['version' => '2.0', 'operator' => '']
                 ]
               ]
-            ]
+            ],
+            ['testSpecificExtensionVersion',
+                [
+                    'extension_versions' => ['testExt' => ['version' => '1.8.0', 'operator' => '']],
+                    'extensions'         => ['testExt']
+                ]
+            ],
+            ['testPHPVersionOperatorLessThan',
+                [
+                    'PHP' => ['version' => '5.4', 'operator' => '<']
+                ]
+            ],
+            ['testPHPVersionOperatorLessThanEquals',
+                [
+                    'PHP' => ['version' => '5.4', 'operator' => '<=']
+                ]
+            ],
+            ['testPHPVersionOperatorGreaterThan',
+                [
+                    'PHP' => ['version' => '99', 'operator' => '>']
+                ]
+            ],
+            ['testPHPVersionOperatorGreaterThanEquals',
+                [
+                    'PHP' => ['version' => '99', 'operator' => '>=']
+                ]
+            ],
+            ['testPHPVersionOperatorEquals',
+                [
+                    'PHP' => ['version' => '5.4', 'operator' => '=']
+                ]
+            ],
+            ['testPHPVersionOperatorDoubleEquals',
+                [
+                    'PHP' => ['version' => '5.4', 'operator' => '==']
+                ]
+            ],
+            ['testPHPVersionOperatorBangEquals',
+                [
+                    'PHP' => ['version' => '99', 'operator' => '!=']
+                ]
+            ],
+            ['testPHPVersionOperatorNotEquals',
+                [
+                    'PHP' => ['version' => '99', 'operator' => '<>']
+                ]
+            ],
+            ['testPHPVersionOperatorNoSpace',
+                [
+                    'PHP' => ['version' => '99', 'operator' => '>=']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorLessThan',
+                [
+                    'PHPUnit' => ['version' => '1.0', 'operator' => '<']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorLessThanEquals',
+                [
+                    'PHPUnit' => ['version' => '1.0', 'operator' => '<=']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorGreaterThan',
+                [
+                    'PHPUnit' => ['version' => '99', 'operator' => '>']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorGreaterThanEquals',
+                [
+                    'PHPUnit' => ['version' => '99', 'operator' => '>=']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorEquals',
+                [
+                    'PHPUnit' => ['version' => '1.0', 'operator' => '=']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorDoubleEquals',
+                [
+                    'PHPUnit' => ['version' => '1.0', 'operator' => '==']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorBangEquals',
+                [
+                    'PHPUnit' => ['version' => '99', 'operator' => '!=']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorNotEquals',
+                [
+                    'PHPUnit' => ['version' => '99', 'operator' => '<>']
+                ]
+            ],
+            ['testPHPUnitVersionOperatorNoSpace',
+                [
+                    'PHPUnit' => ['version' => '99', 'operator' => '>=']
+                ]
+            ],
+            ['testExtensionVersionOperatorLessThanEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '1.0', 'operator' => '<=']]
+                ]
+            ],
+            ['testExtensionVersionOperatorGreaterThan',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '99', 'operator' => '>']]
+                ]
+            ],
+            ['testExtensionVersionOperatorGreaterThanEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '99', 'operator' => '>=']]
+                ]
+            ],
+            ['testExtensionVersionOperatorEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '1.0', 'operator' => '=']]
+                ]
+            ],
+            ['testExtensionVersionOperatorDoubleEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '1.0', 'operator' => '==']]
+                ]
+            ],
+            ['testExtensionVersionOperatorBangEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '99', 'operator' => '!=']]
+                ]
+            ],
+            ['testExtensionVersionOperatorNotEquals',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '99', 'operator' => '<>']]
+                ]
+            ],
+            ['testExtensionVersionOperatorNoSpace',
+                [
+                    'extensions'         => ['testExtOne'],
+                    'extension_versions' => ['testExtOne' => ['version' => '99', 'operator' => '>=']]
+                ]
+            ],
         ];
     }
 
@@ -177,8 +328,8 @@ class Util_TestTest extends PHPUnit_Framework_TestCase
     public function testGetRequirementsMergesClassAndMethodDocBlocks()
     {
         $expectedAnnotations = [
-            'PHP'       => '5.4',
-            'PHPUnit'   => '3.7',
+            'PHP'       => ['version' => '5.4', 'operator' => ''],
+            'PHPUnit'   => ['version' => '3.7', 'operator' => ''],
             'OS'        => '/WINNT/i',
             'functions' => [
               'testFuncClass',
@@ -214,18 +365,40 @@ class Util_TestTest extends PHPUnit_Framework_TestCase
             ['testOne',            []],
             ['testNine',           ['Function testFunc is required.']],
             ['testTen',            ['Extension testExt is required.']],
-            ['testAlwaysSkip',     ['PHPUnit 1111111 (or later) is required.']],
-            ['testAlwaysSkip2',    ['PHP 9999999 (or later) is required.']],
+            ['testAlwaysSkip',     ['PHPUnit >= 1111111 is required.']],
+            ['testAlwaysSkip2',    ['PHP >= 9999999 is required.']],
             ['testAlwaysSkip3',    ['Operating system matching /DOESNOTEXIST/i is required.']],
             ['testAllPossibleRequirements', [
-              'PHP 99-dev (or later) is required.',
-              'PHPUnit 9-dev (or later) is required.',
+              'PHP >= 99-dev is required.',
+              'PHPUnit >= 9-dev is required.',
               'Operating system matching /DOESNOTEXIST/i is required.',
               'Function testFuncOne is required.',
               'Function testFuncTwo is required.',
               'Extension testExtOne is required.',
               'Extension testExtTwo is required.',
+              'Extension testExtThree >= 2.0 is required.',
             ]],
+            ['testPHPVersionOperatorLessThan', ['PHP < 5.4 is required.']],
+            ['testPHPVersionOperatorLessThanEquals', ['PHP <= 5.4 is required.']],
+            ['testPHPVersionOperatorGreaterThan', ['PHP > 99 is required.']],
+            ['testPHPVersionOperatorGreaterThanEquals', ['PHP >= 99 is required.']],
+            ['testPHPVersionOperatorNoSpace', ['PHP >= 99 is required.']],
+            ['testPHPVersionOperatorEquals', ['PHP = 5.4 is required.']],
+            ['testPHPVersionOperatorDoubleEquals', ['PHP == 5.4 is required.']],
+            ['testPHPUnitVersionOperatorLessThan', ['PHPUnit < 1.0 is required.']],
+            ['testPHPUnitVersionOperatorLessThanEquals', ['PHPUnit <= 1.0 is required.']],
+            ['testPHPUnitVersionOperatorGreaterThan', ['PHPUnit > 99 is required.']],
+            ['testPHPUnitVersionOperatorGreaterThanEquals', ['PHPUnit >= 99 is required.']],
+            ['testPHPUnitVersionOperatorEquals', ['PHPUnit = 1.0 is required.']],
+            ['testPHPUnitVersionOperatorDoubleEquals', ['PHPUnit == 1.0 is required.']],
+            ['testPHPUnitVersionOperatorNoSpace', ['PHPUnit >= 99 is required.']],
+            ['testExtensionVersionOperatorLessThan', ['Extension testExtOne < 1.0 is required.']],
+            ['testExtensionVersionOperatorLessThanEquals', ['Extension testExtOne <= 1.0 is required.']],
+            ['testExtensionVersionOperatorGreaterThan', ['Extension testExtOne > 99 is required.']],
+            ['testExtensionVersionOperatorGreaterThanEquals', ['Extension testExtOne >= 99 is required.']],
+            ['testExtensionVersionOperatorEquals', ['Extension testExtOne = 1.0 is required.']],
+            ['testExtensionVersionOperatorDoubleEquals', ['Extension testExtOne == 1.0 is required.']],
+            ['testExtensionVersionOperatorNoSpace', ['Extension testExtOne >= 99 is required.']],
         ];
     }
 
