@@ -46,4 +46,31 @@ class Util_XMLTest extends PHPUnit_Framework_TestCase
 
         return $data;
     }
+
+    /**
+     * @expectedException PHPUnit_Framework_Exception
+     * @expectedExceptionMessage Could not load XML from empty string
+     */
+    public function testLoadEmptyString()
+    {
+        PHPUnit_Util_XML::load('');
+    }
+
+    /**
+     * @expectedException PHPUnit_Framework_Exception
+     * @expectedExceptionMessage Could not load XML from array
+     */
+    public function testLoadArray()
+    {
+        PHPUnit_Util_XML::load([1, 2, 3]);
+    }
+
+    /**
+     * @expectedException PHPUnit_Framework_Exception
+     * @expectedExceptionMessage Could not load XML from boolean
+     */
+    public function testLoadBoolean()
+    {
+        PHPUnit_Util_XML::load(false);
+    }
 }
