@@ -5,14 +5,13 @@
  * with some minor changes
  * This class required for PHPUnit_Framework_Constraint_ArrayHasKey testing
  */
-
 class SampleArrayAccess implements ArrayAccess
 {
     private $container;
 
     public function __construct()
     {
-        $this->container = array();
+        $this->container = [];
     }
     public function offsetSet($offset, $value)
     {
