@@ -14,7 +14,7 @@ class Singleton
     public static function getInstance()
     {
         if (self::$uniqueInstance === null) {
-            self::$uniqueInstance = new Singleton;
+            self::$uniqueInstance = new self;
         }
 
         return self::$uniqueInstance;
